"""
$RCSfile: CacheManager.py,v $

XMLTransform automatically transforms an XML document via XSLT, where
the XML document is obtained from another Zope object via acquisition.
CacheManager encapsulates information about a logically
related group of XSLT transformers (aka stylesheets).

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/27 23:07:32 $'[6:-2]
__version__ = '$Revision: 1.15 $'[10:-2]

#
# Add our base classes.  Derived classes will get this stuff automatically
#
from OFS.SimpleItem import SimpleItem  # The SimpleItem module is a Zope builtin that provides
                                       # cut/paste, management views, webdav/ftp, undo, ownership,
                                       # traversal, acquisition, and persistence
from Products.ZCatalog.CatalogAwareness import CatalogAware # Reqd for ZCatalog support
from OFS.PropertyManager import PropertyManager  # Reqd for proper property mgmt

# peer classes
from ICacheManager import ICacheManager
import XMLTransform # the module

# Zope builtins
from Globals import Acquisition, DTMLFile, MessageDialog, HTML
from Acquisition import aq_chain, aq_parent
from AccessControl import ClassSecurityInfo
import OFS      # we grab his _property settings
import Globals  # InitializeClass (security stuff)

# Python builtins
import os, os.path, sys, urlparse

################################################################
# Defaults
################################################################

cachePrefix    = 'ZopeCache'

def defaultCachePrefix():
    spoolDirectory = "/tmp"
    if sys.platform == 'win32':
        # use forward slash b/c backslash is special char        
        spoolDirectory = "c:/tmp" 
    return spoolDirectory + "/" + cachePrefix

################################################################
# Module Scoped Convenience Methods
################################################################

def addInstance(folder, id, title='', description=''):
    """
    
    This is a convenience factory method for creating an instance of
    CacheManager.  It returns the object created, and may therefore be
    more convenient than the addCacheManager() method it calls.  It is
    used by the unit testing programs.

    """
    folder.manage_addProduct['XMLTransform'].addXMLTransformCacheManager(id,title,description)
    return folder[id]

################################################################
# Container Methods
################################################################

# globals() is standard python for passing the current namespace
manage_addXMLTransformCacheManagerForm = DTMLFile('dtml/create_cachemgr', globals())

def manage_addXMLTransformCacheManager(self,
                                       id, title='', description='',
                                       REQUEST=None, RESPONSE=None):

    """
    
    Add an XMLTransformer CacheManager to a folder.  Called from the
    create_cachemgr.dtml GUI form in the Zope Management interface.
    It calls addXMLTransformCacheManager to actually do the work.

    """
    try:
        self.addXMLTransformCacheManager(id, title, description)
        message = 'successfully created ' + id + ' cache manager object.'        
        return MessageDialog(title   = 'CREATED',
                             message = message,
                             action  = 'manage_main')
    except Exception, e:
        message = str(e)
        mesage.replace('\n','<br/>')
        return MessageDialog(title   = 'ERROR',
                             message = message,
                             action  = 'manage_main')

def addXMLTransformCacheManager(self, id, title='', description=''):

    """
    
    Add an XMLTransformer CacheManager to a folder.  You should call
    this method directly if you are creating an instance of
    CacheManager programmatically.

    """
    if not id:
        raise Exception('Required fields must not be blank')

    self._setObject(id, CacheManager(id, title, description))

################################################################
# CacheManager class
################################################################

class CacheManager(CatalogAware,    # ZCatalog support
                   PropertyManager, # Property support
                   SimpleItem):
    """
    CacheManager
    """

    meta_type = 'XML Transform Cache Manager'  # This is the name Zope will use for the Product in
    # the "addProduct" list

    __implements__ = ICacheManager

    # This tuple defines our properties
    _properties = (
        {'id':'title',           'type':'string',    'mode': 'w'},
        {'id':'description',     'type':'string',    'mode': 'w'},
        {'id':'cachePrefix',     'type':'string',    'mode': 'w'},        
        )

    # This tuple defines a dictionary for each tab in the management interface
    # label = label of tab, action = url it links to
    manage_options = (
        {'label':'Cache',    'action':'manage_cacheForm',    'help': ('XMLTransform','cache.stx') },
        ) + OFS.PropertyManager.PropertyManager.manage_options + OFS.SimpleItem.SimpleItem.manage_options

    _security = ClassSecurityInfo()

    # set security for the object itself, e.g. if it is accessed in
    # DTML code This line is REQUIRED to allow access to XMLTransforms
    # by the public (ariel DTDs, etc.)
    _security.declareObjectPublic()

    # call __call__() method on DTMLFile for our edit form, point at our edit_form.dtml
    # globals() is standard python for passing the current namespace
    _security.declareProtected('Manage XML Transform Cache',      'manage_cacheForm')
    manage_cacheForm           = DTMLFile('dtml/cache_form',globals())

    def __init__(self, id, title, description):
        """
        Initialize a new instance of CacheManager
        """
        self.id             = id
        self.title          = title
        self.description    = description
        self.__version__    = __version__

        self.cachePrefix    = defaultCachePrefix()

        self.cacheFiles     = {}

    ################################################################
    # Methods called from ZMI pages
    ################################################################

    _security.declareProtected('Edit','manage_setCachingOn')
    def manage_setCachingOn(self, REQUEST):
        "Cover for calling from DTML form in ZMI"

        message = self.batchSetCachingOn(REQUEST)
        message.replace('\n','<br/>')        
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_setCachingOff')
    def manage_setCachingOff(self, REQUEST):
        "Cover for calling from DTML form in ZMI"

        message = self.batchSetCachingOff(REQUEST)
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_regenerateAll')
    def manage_regenerateAll(self, REQUEST):
        "Cover for calling from DTML form in ZMI"

        message = self.regenerateAll(REQUEST)
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_regenerateIfNeeded')
    def manage_regenerateIfNeeded(self, REQUEST):
        "Cover for calling from DTML form in ZMI"

        message = self.regenerateIfNeeded(REQUEST)
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_clearCache')
    def manage_clearCache(self):
        "Cover for calling from DTML form in ZMI"

        message = self.clearCache()
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_listCacheFiles')
    def manage_listCacheFiles(self):
        "Cover for calling from DTML form in ZMI"

        message = "Cache Files:\n\n"
        for file in self.listCacheFiles():
            message = message + file + '\n'
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_listUpToDate')
    def manage_listUpToDate(self):
        "Cover for calling from DTML form in ZMI"

        message = "Up To Date XMLTransforms:\n\n"
        for ob in self.listUpToDate():
            message = message + ob + '\n'
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit','manage_listOutOfDate')
    def manage_listOutOfDate(self):
        "Cover for calling from DTML form in ZMI"

        message = "Out Of Date XMLTransforms:\n\n"
        for ob in self.listOutOfDate():
            message = message + ob + '\n'
        message.replace('\n','<br/>')
        return self.manage_cacheForm(batchOperationOutput = message)

    _security.declareProtected('Edit', 'manage_editProperties')
    def manage_editProperties(self, REQUEST):
        """

        Cover for PropertyManager.manage_editProperties() method.
        First validate cachePrefix, then pass it on to the inherited
        method for further processing

        """
        for prop in self._propertyMap():
            name = prop['id']
            if name == 'cachePrefix':
                value = REQUEST.get(name, '')
                if not self.isValidCachePrefix(value):
                    return MessageDialog(title   = 'ERROR',
                                         message = value + ' is not a valid Cache Prefix',
                                         action  = 'manage_propertiesForm')
        return CacheManager.inheritedAttribute("manage_editProperties")(self, REQUEST)

    ################################################################
    # Methods implementing the ICacheManager interface
    ################################################################

    _security.declareProtected('Edit','batchSetCachingOn')
    def batchSetCachingOn(self, REQUEST):
        """

        Turns caching on for all instances of XMLTransform within the
        scope of this cache manager.

        """
        parent = self.getParentFolder()
        message = "Turning Caching On for all XMLTransform instances under " + \
                  parent.getId() + \
                  " Folder:\n"
        message = message + \
                  self.batchExecute(parent, 'setCachingOn', REQUEST)
        return message

    _security.declareProtected('Edit', 'batchSetCachingOff')
    def batchSetCachingOff(self, REQUEST):
        """

        Turns caching off for all instances of XMLTransform within the
        scope of this cache manager.


        """
        parent = self.getParentFolder()
        message = "Turning Caching Off for all XMLTransform instances under " + \
                  parent.getId() + \
                  " Folder:\n"
        message = message + \
                  self.batchExecute(parent, 'setCachingOff', REQUEST)
        return message

    _security.declareProtected('Edit', 'regenerateAll')
    def regenerateAll(self, REQUEST):
        """

        Forces all instances of XMLTransform within the scope of this
        cache manager to regenerate their contents.  They may or may
        not cache the result, depending on their cache settings.

        """
        parent = self.getParentFolder()
        message = "Forcing transformation for all XMLTransform instances under " + \
                  parent.getId() + \
                  " Folder:\n"
        message = message + \
                  self.batchExecute(parent, 'transformAndCache', REQUEST)
        return message

    _security.declareProtected('Edit', 'regenerateIfNeeded')
    def regenerateIfNeeded(self, REQUEST):
        """

        For all instances of XMLTransform within the scope of this
        cache manager that have caching on, check to see if the cached
        version of their contents is up to date.  If not, regenerate
        it and cache it.  This method does not affect those instances
        of XMLTransform for which caching is set to off.

        """
        parent = self.getParentFolder()
        message = "Running transformation for all XMLTransform instances under " + \
                  parent.getId() + \
                  "\nFolder for which the cache is out of date:\n\n"
        parent = self.getParentFolder()
        objects = parent.ZopeFind(parent, obj_metatypes=['XML Transform'],
                                  search_sub=1)
        for doc in objects:
            try:
                message = message + "\nXMLTransform: " + doc[0]
                if doc[1].isCacheFileUpToDate():
                    message = message + " is up to date."
                else:
                    message = message + " is out of date"
                    doc[1].transformAndCache(REQUEST)
                    message = message + "...updated and cached."
            except Exception, value:
                message = message + "...error: %s" % (value)
        return message

    _security.declareProtected('Edit', 'cacheFileTimeStamp')
    def cacheFileTimeStamp(self, client):
        """
        
        Return the last modified time of the cache file for the passed in
        XMLTransform client object, or 0 if the cache file does not exist

        """
        fileName  = self.cacheFileName(client)
        cachetime = 0

        if os.path.exists(fileName):
            return os.path.getmtime(fileName)
        else:
            return 0

    _security.declareProtected('Edit', 'valueFromCache')
    def valueFromCache(self, client):
        """

        Retrieve the output from the cache for the passed in
        XMLTransform client object, or None if the cache file does not
        exist.

        """
        fileName = self.cacheFileName(client)
        cache    = open(fileName, 'rb')
        result   = cache.read()
        cache.close()
        return result

    _security.declareProtected('Edit', 'saveToCache')
    def saveToCache(self, client, aString):
        """

        Save the contents (a string) to the cache for the passed in
        XMLTransform client object.  It will overwrite the previous
        contents of the cache file, or create a new cache file if it
        does not yet exist.

        """
        objectname = client.getPhysicalPath()
        filename  = self.cacheFileName(client)
        cache     = open(filename, 'wb')
        cache.write(aString)
        cache.close()
        # save value in cacheFiles map
        self.cacheFiles[objectname] = filename
        self._p_changed = 1  # dirty flag, triggers persistence to update the object        

    _security.declareProtected('Edit', 'clearCache')
    def clearCache(self):
        """

        Clear the cache.  This involves removing every cache file from
        the file system.

        """
        message = ""
        for filename in self.cacheFiles.values():
            try:
                os.remove(filename)
                message = message + filename + " successfully removed\n"
            except Exception, value:
                message = message + "%s not removed, error: %s\n" % ( filename, value )
        self.cacheFiles.clear()
        self._p_changed = 1  # dirty flag, triggers persistence to update the object
        return message or "No files in cache"

    _security.declareProtected('Edit', 'listCacheFiles')
    def listCacheFiles(self):
        """
        
        Return the list of files in the cache

        """
        return self.cacheFiles.values()

    def listUpToDate(self):
        """

        List all XMLTransform instances within the scope of this Cache
        Manager that are up to date with respect to this cache.

        """
        upToDaters = []
        
        parent = self.getParentFolder()
        objects = parent.ZopeFind(parent, obj_metatypes=['XML Transform'],
                                  search_sub=1)
        for doc in objects:
            try:
                if doc[1].isCacheFileUpToDate():
                    upToDaters.append(doc[0])
            except:
                pass
        return upToDaters

    def listOutOfDate(self):
        """

        List all XMLTransform instances within the scope of this Cache
        Manager that are out to date with respect to this cache.

        """
        outOfDaters = []
        
        parent = self.getParentFolder()
        objects = parent.ZopeFind(parent, obj_metatypes=['XML Transform'],
                                  search_sub=1)
        for doc in objects:
            try:
                if not doc[1].isCacheFileUpToDate():
                    outOfDaters.append(doc[0])
            except:
                pass
        return outOfDaters

    ################################################################
    # Utilities
    ################################################################

    def isValidCachePrefix(self, value):
        "Return 1 if this represents a valid value for a cache prefix"
        dirname = os.path.dirname(value)
        
        # We would like to check write permission in the directory
        # also, but this is difficult to do portably... TBD

        return os.path.exists(dirname)
    
    def cacheFileName(self, client):
        """
        
        Algorithmically determine the leaf name of the file in which
        to store the transformed contents of the passed in
        XMLTransform client object.

        """
    
        ( addressingScheme, networkLocation, path, params, query, fragment ) = \
          urlparse.urlparse(client.absolute_url())
        return os.path.normpath(self.cachePrefix + path.replace("/","_"))

    def getParentFolder(self):
        "Returns the folder containing this instance of CacheManager"
        return aq_parent(self)

    def batchExecute(self, context, methodName, REQUEST):
        """
        
        Recursively execute passed-in method on all objects of type
        XMLTransform within the passed-in context (ie Zope folder).
        Returns a list of the object IDs of the affected objects.

        """

        metaType = 'XML Transform'
        objects = context.ZopeFind(context, obj_metatypes=[metaType],
                                   search_sub=1)
        func = XMLTransform.XMLTransform.__dict__[methodName]
        message = "\n\n"

        for doc in objects:
            try:
                message = message + "\nFound XMLTransform " + \
                          doc[0]
                func(doc[1], REQUEST)
                message = message + "...success"
            except Exception, value:
                mesage = message + "...error: %s" % (value)
                
        return message

    # innocuous methods should be declared public
    _security.declarePublic('getSelf')
    def getSelf(self):
        "Return this object. For use in DTML scripts"
        return self.aq_chain[0]

    ################################################################
    # Standard Zope stuff
    ################################################################

    _security.declareProtected('View', '__str__')
    def __str__(self, REQUEST = None):
        "return contents as string"
        message = self.id + " (" + self.title + ")"
        return message

    ################################################################
    # Support for Schema Migration
    ################################################################

    def repair(self):
        """

        Repair this object.  This method is used for schema migration,
        when the class definition changes and existing instances of
        previous versions must be updated.

        """
        #if not hasattr(self,'_caching'):
        #    self._caching        = 'on'
        if not hasattr(self,'_fileModTime'):
            self._fileModTime    = 0

        #self.caching    = 'on'
        #self.debugLevel = 0
        
        #if not hasattr(self, 'behave_like'):
        #    self.behave_like = 'DTMLDocument'

# register security information
Globals.InitializeClass(CacheManager)
