"""
$RCSfile: ICacheManager.py,v $

XMLTransform automatically transforms an XML document via XSLT, where
the XML document is obtained from another Zope object via acquisition.
ICacheManager is the Interface implemented by CacheManager.


Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/27 22:57:56 $'[6:-2]
__version__ = '$Revision: 1.6 $'[10:-2]

# Zope Base Classes
from Interface import Base

################################################################
# ICacheManager interface
################################################################

class ICacheManager(Base):
    """

    XML Transform Cache Manager public interface

    """

    def batchSetCachingOn(self):
        """

        Turns caching on for all instances of XMLTransform within the
        scope of this cache manager.

        """

    def batchSetCachingOff(self):
        """

        Turns caching off for all instances of XMLTransform within the
        scope of this cache manager.


        """

    def regenerateAll(self):
        """

        Forces all instances of XMLTransform within the scope of this
        cache manager to regenerate (transform) their contents.  They
        may or may not cache the result, depending on their cache
        settings.

        """

    def regenerateIfNeeded(self):
        """

        For all instances of XMLTransform within the scope of this
        cache manager that have caching on, check to see if the cached
        version of their contents is up to date.  If not, regenerate
        it and cache it.  This method does not affect those instances
        of XMLTransform for which caching is set to off.

        """

    def cacheFileTimeStamp(self, client):
        """
        
        Return the last modified time of the cache file for the passed in
        XMLTransform client object, or 0 if the cache file does not exist

        """

    def valueFromCache(self, client):
        """

        Retrieve the output from the cache for the passed in
        XMLTransform client object, or None if the cache file does not
        exist.

        """

    def saveToCache(self, client, aString):
        """

        Save some contents (a string) to the cache for the passed in
        XMLTransform client object.  It will overwrite the previous
        contents of the cache file, or create a new cache file if it
        does not yet exist.

        """

    def clearCache(self):
        """

        Clear the cache.  This involves removing every cache file from
        the file system.

        """

    def listCacheFiles(self):
        """

        Return the list of files in the cache

        """

    def listUpToDate(self):
        """

        List all XMLTransform instances within the scope of this Cache
        Manager that are up to date with respect to this cache.

        """

    def listOutOfDate(self):
        """

        List all XMLTransform instances within the scope of this Cache
        Manager that are out to date with respect to this cache.

        """
